<?php

// Include the configuration file
include_once 'config-%_weaverfm-uniqueid_%.php';

include_once('FileManager.php');
require_once(FileManagerUtility::getPath() . '/Image.php');
require_once(FileManagerUtility::getPath() . '/WFMUpload.php');

// This class extends FileManager declared in FileManager.php
class WFMFileManager extends FileManager {
	
	// Called when a user tries to download a file
	// This is added to initiate the browser's file download dialog
	protected function onDownload() {
		
		// If the file and directory are set and the user is authorized to download the file
		if(isset($this->get['file']) && 
			isset($this->get['directory']) &&
			isset($this->get['format']) &&
			(function_exists('DownloadIsAuthenticated') && DownloadIsAuthenticated($this->get))) {
			
			// Get the real directory
			$dir = $this->getDir((FileManagerUtility::startsWith($this->get['directory'], $this->basename) ? '' : $this->basename).$this->get['directory'].'/');
			
			// The file for the request
			$file = $dir.'/'.$this->get['file'];
			
			// Get the format of the download
			$fmt = $this->get['format'];
			
			// Check to make sure the file to download is not a directory
			if(!is_dir($file)){				
				// Check to see if a file exists at the path
				if(file_exists($file)) {
					
					// The basename
					$basename = basename($file);
					
					// Turn off error reporting
					error_reporting(0);
					
					// If the user specified the raw file download
					if(strcasecmp($fmt, 'raw') === 0) {
					
						// Should we use HTML redirect?
						global $html_redirect;
						if ($html_redirect) {
						
							$path = '../'.(empty($this->options['baseURL']) ? '' : $this->options['baseURL']).$this->get['directory'].$this->get['file'];
							
							// Default to a RAW file download
							header('Location: '.$path);
						} else
						{
							// Default to a RAW file download
							ob_start();
							header('Content-Description: File Transfer');
							header('Content-Type: application/'.$this->getMimeType($file));
							header('Content-Disposition: attachment; filename='.$basename);
							header('Content-Transfer-Encoding: binary');
							header('Expires: 0');
							header('Cache-Control: must-revalidate, post-check=0, pre-check=0');
							header('Pragma: public');
							header('Content-Length: ' . filesize($file));
							ob_clean();
						
							// Get the timeout time
							$timeout = ini_get('max_execution_time');
							
							// The size of 1 chunk of data (in bytes)
							$chunk_size = 1024;
						
							// Loop over the file and read it in chunks. This way we support resuming file transfers
							$fp = fopen( $file, 'rb');
							while(!feof($fp)) {
								
								// Read in X bytes
								echo(fread($fp, $chunk_size));
								
								// Flush the buffers
								ob_flush();
								flush();
								
								// Look, we sent data, reset the timeout
								set_time_limit($timeout);
							}
							
							// Close the file
							fclose($fp);
						}
						
						// If we get here the download completed successfully. Notify any callbacks
						if (function_exists('DownloadComplete')) {
							// The file extension
							$ext = strtolower(pathinfo($basename, PATHINFO_EXTENSION));
							$name = pathinfo_filename($basename);
							DownloadComplete($this->get['session_username'], $name, $ext);
						}
					}
				}
			}
		}
	}
	
	// These method are the same as the one implemented in FileManager.php EXCEPT it uses
	// the WFMUpload subclass
	
	protected function onUpload(){
		try{
			if (!$this->options['upload'])
				throw new FileManagerException('disabled');
			if (empty($this->get['directory']) || (function_exists('UploadIsAuthenticated') && !UploadIsAuthenticated($this->get)))
				throw new FileManagerException('authenticated');

			$dir = $this->getDir((FileManagerUtility::startsWith($this->get['directory'], $this->basename) ? '' : $this->basename).$this->get['directory'].'/');
			
			$name = pathinfo_filename(NULL);
			if (WFMUpload::exists('Filedata')) {
				$name = pathinfo_filename($this->getName($_FILES['Filedata']['name'], $dir));
			}
						
			// The file extension
			$ext = strtolower(pathinfo($_FILES['Filedata']['name'], PATHINFO_EXTENSION));
			
			$file = WFMUpload::move('Filedata', $dir . '/', array(
				'name' => $name,
				'extension' => $this->options['safe'] && $name && in_array($ext, array('exe', 'dll', 'php', 'php3', 'php4', 'php5', 'phps')) ? 'txt' : null,
				'size' => $this->options['maxUploadSize'],
				'mimes' => $this->getAllowedMimeTypes()
			));

			if (FileManagerUtility::startsWith(WFMUpload::mime($file), 'image/') && !empty($this->get['resize'])){
				$img = new Image($file);
				$size = $img->getSize();
				if ($size['width'] > 800) $img->resize(800)->save();
				elseif ($size['height'] > 600) $img->resize(null, 600)->save();
			}

			echo json_encode(array(
				'status' => 1,
				'name' => pathinfo($file, PATHINFO_BASENAME)
			));
			
			// If we get here the upload completed successfully. Notify any callbacks
			if (function_exists('UploadComplete')) {
				UploadComplete($this->get['session_username'], $name, $ext);
			}
		}
		catch(UploadException $e){
			echo json_encode(array(
				'status' => 0,
				'error' => class_exists('ValidatorException') ? strip_tags($e->getMessage()) : '${upload.' . $e->getMessage() . '}' // This is for Styx :)
			));
		}
		catch(FileManagerException $e){
			echo json_encode(array(
				'status' => 0,
				'error' => '${upload.' . $e->getMessage() . '}'
			));
		}
	}
	
	// This method is exactly the same as the base class except it uses pathinfo_filename
	/* This method is used by both move and rename */
	protected function onMove(){
		if (empty($this->post['directory']) || empty($this->post['file'])) return;
		
		$rename = empty($this->post['newDirectory']) && !empty($this->post['name']);
		$dir = $this->getDir($this->post['directory']);
		$file = realpath($dir . '/' . $this->post['file']);
		
		$is_dir = is_dir($file);
		if (!$this->checkFile($file) || (!$rename && $is_dir))
			return;
		
		if ($rename || $is_dir){
			if (empty($this->post['name'])) return;
			$newname = $this->getName($this->post['name'], $dir);
			$fn = 'rename';
		}else{
			$newname = $this->getName(pathinfo_filename($file), $this->getDir($this->post['newDirectory']));
			$fn = !empty($this->post['copy']) ? 'copy' : 'rename';
		}
		
		if (!$newname) return;
		
		$ext = pathinfo($file, PATHINFO_EXTENSION);
		if ($ext) $newname .= '.' . $ext;
		$fn($file, $newname);
		
		echo json_encode(array(
			'name' => pathinfo($this->normalize($newname), PATHINFO_BASENAME),
		));
	}
	
	// This method is exactly the same as the base class except it uses pathinfo_filename
	protected function getName($file, $dir){
		$files = array();
		foreach ((array)glob($dir . '/*') as $f)
			$files[] = pathinfo_filename($f);
		
		$pathinfo = pathinfo($file);
		$file = $dir . '/' . FileManagerUtility::pagetitle(pathinfo_filename($file), $files).(!empty($pathinfo['extension']) ? '.' . $pathinfo['extension'] : null);
		
		return !$file || !FileManagerUtility::startsWith($file, $this->basedir) || file_exists($file) ? null : $file;
	}
	
	protected function getMimeType($file){
		return is_dir($file) ? 'text/directory' : WFMUpload::mime($file);
	}
	
	// This method is the same as the base class EXCEPT is remove the Icons/ from the img path
	protected function onDetail(){
		if (empty($this->post['directory']) || empty($this->post['file'])) return;
		
		$file = realpath($this->path . '/' . $this->post['directory'] . '/' . $this->post['file']);
		if (!$this->checkFile($file)) return;
		
		require_once($this->options['id3Path']);
		
		$url = $this->options['baseURL'] . $this->normalize(substr($file, strlen($this->path)+1));
		$mime = $this->getMimeType($file);
		$content = null;
		if (FileManagerUtility::startsWith($mime, 'image/')){
			$size = getimagesize($file);
			$content = '<img src="' . $url . '" class="preview" alt="" />
				<h2>${more}</h2>
				<dl>
					<dt>${width}</dt><dd>' . $size[0] . 'px</dd>
					<dt>${height}</dt><dd>' . $size[1] . 'px</dd>
				</dl>';
		}elseif (FileManagerUtility::startsWith($mime, 'text/') || $mime == 'application/x-javascript'){
			$filecontent = file_get_contents($file, null, null, 0, 300);
			if (!FileManagerUtility::isBinary($filecontent)) $content = '<div class="textpreview">' . nl2br(str_replace(array('$', "\t"), array('&#36;', '&nbsp;&nbsp;'), htmlentities($filecontent))) . '</div>';
		}elseif ($mime == 'application/zip'){
			$out = array(array(), array());
			$getid3 = new getID3();
			$getid3->Analyze($file);
			foreach ($getid3->info['zip']['files'] as $name => $size){
				$icon = is_array($size) ? 'dir' : $this->getIcon($name);
				$out[($icon == 'dir') ? 0 : 1][$name] = '<li><a><img src="' . $this->options['assetBasePath'] . '/' . $icon . '.png" alt="" /> ' . $name . '</a></li>';
			}
			natcasesort($out[0]);
			natcasesort($out[1]);
			$content = '<ul>' . implode(array_merge($out[0], $out[1])) . '</ul>';
		}elseif (FileManagerUtility::startsWith($mime, 'audio/')){
			$getid3 = new getID3();
			$getid3->Analyze($file);
			
			$content = '<div class="object">
					<object type="application/x-shockwave-flash" data="' . $this->options['assetBasePath'] . '/dewplayer.swf?mp3=' . rawurlencode($url) . '&volume=30" width="200" height="20">
						<param name="movie" value="' . $this->options['assetBasePath'] . '/dewplayer.swf?mp3=' . rawurlencode($url) . '&volume=30" />
					</object>
				</div>
				<h2>${more}</h2>
				<dl>
					<dt>${title}</dt><dd>' . $getid3->info['comments']['title'][0] . '</dd>
					<dt>${artist}</dt><dd>' . $getid3->info['comments']['artist'][0] . '</dd>
					<dt>${album}</dt><dd>' . $getid3->info['comments']['album'][0] . '</dd>
					<dt>${length}</dt><dd>' . $getid3->info['playtime_string'] . '</dd>
					<dt>${bitrate}</dt><dd>' . round($getid3->info['bitrate']/1000) . 'kbps</dd>
				</dl>';
		}
		$encoded_download_url = $this->options['assetBasePath'].$this->options['scriptPath'].'?event=download&file='.$this->post['file'].'&directory='.$this->post['directory'].'&format=raw';
		echo json_encode(array(
			'content' => $content ? $content : '<div class="margin">
				${nopreview}<br/><button value="' . $encoded_download_url . '">${download}</button>
			</div>'
		));
	}
	
	// This method is the same as in the base class except it checks for upload permissions
	protected function onCreate(){
		if ((function_exists('CreateIsAuthenicated') && !CreateIsAuthenicated($this->get))) {
			throw new FileManagerException('authenticated');
		} else {
			/*parent::onCreate();*/
			if (empty($this->post['directory']) || empty($this->post['file'])) return;

			$file = $this->getName($this->post['file'], $this->getDir($this->post['directory']));
			if (!$file) return;

			mkdir($file);
			chmod($file, 0777);

			$this->onView();
		}
	}
	
	// This method is the same as in the base class except it checks for destroy permissions
	protected function onDestroy(){
		try{
			if ((function_exists('DestroyIsAuthenticated') && !DestroyIsAuthenticated($this->get))) {
				throw new FileManagerException('authenticated');
			} else {
				parent::onDestroy();
			}
		} catch(FileManagerException $e){
			echo json_encode(array(
				'status' => 0,
				'error' => '${upload.' . $e->getMessage() . '}'
			));
		}
	}
	
	// This method is the same as in the base class except it checks for view permissions
	protected function onView(){
		try{
			if ((function_exists('ViewIsAuthenticated') && !ViewIsAuthenticated($this->get))) {
				throw new FileManagerException('authenticated');
			} else {
				parent::onView();
			}
		} catch(FileManagerException $e){
			echo json_encode(array(
				'status' => 0,
				'error' => '${upload.' . $e->getMessage() . '}'
			));
		}
	}
}

// For PHP 5.x support
// Author: Gregory Barchard
// February 23, 2010
// Based on work done at http://www.php.net/manual/en/function.pathinfo.php#88159
function pathinfo_filename($file) { //file.name.ext, returns file.name

	// If the PATHINFO_FILENAME variable is defined, use it
    if (defined('PATHINFO_FILENAME')) return pathinfo($file, PATHINFO_FILENAME);

	// Get the path info for the file
	$pathinfo = pathinfo($file);
	
	// Get the basename (filename.extension)
	$basename = $pathinfo['basename'];
	
	// Assume the filename to be the same as the basename
	$filename = $basename;
	
	// If an extension exists in the basename
    if (strstr($basename, '.')) {
	
		// Split the filename off
		$filename = substr($basename, 0, strrpos($basename, '.'));
	}
	
	// Return the filename
	return $filename;
}
?>